type 'a btree = Br of 'a * 'a btree * 'a btree | Lf;;
type 'a option = Some of 'a | None;;

(* Common auxiliary functions BEGIN *)
let rec take i = function
| [] -> []
| x::xs ->
   if i > 0 then x :: take (i - 1) xs
   else [];;

let rec drop i = function
| [] -> []
| x::xs ->
   if i > 0 then drop (i-1) xs
   else x::xs;;

let is_geq = function
(Br(a, _, _), Br(b, _, _)) -> a >= b
| (Lf, Br(b, _, _)) -> false
| (Br(a, _, _), Lf) -> true;;

let get_val (Br(a, _, _)) = a;;

let max_option = function
  (None, Some b) -> Some b
| (Some a, None) -> Some a
| (Some a, Some b) -> Some (if a >= b then a else b)
| _ -> None;;
(* Common auxiliary functions END *)

(* Constructs a tournament tree for a given list of integers.
   It also places the smaller value on the right, so we can find
   it efficiently when searching for the second max value. *)
let rec find_tournament_tree = function
[] -> Lf
| [x] -> Br(x, Lf, Lf)
| xs -> let left = find_tournament_tree(take((List.length xs) / 2) xs) in 
        let right = find_tournament_tree(drop((List.length xs) / 2) xs)
        in
           if is_geq(left, right) then Br(get_val(left), left, right)
           else Br(get_val(right), right, left);;

(* Find the maximum values among the right branches in the path of the maximum. *)
let rec find_second_best = function
  (Br(_, _, Lf)) -> None
| (Br(_, ell, Br(x, _, _))) -> max_option (find_second_best ell, Some x);;


let tournament_tree = find_tournament_tree [7; 5; 8; 2; 3; 6; 12; 4; 17; 9];;
find_second_best tournament_tree;;
