#include <cstdio>
#include <cstdlib>
#include <algorithm>

using namespace std;

const size_t MAXN = 1'000'000;

struct cow{
   int x, r;
   bool operator<(const cow &C)const{
      return x < C.x;
   }
} K[MAXN];

int n, num_breads;
int A[MAXN];

/* Determines if it is possible to find an interval of length
   at least len which contains all cows. */
bool F(int len){
   // lef: leftmost cow
   int lef = 0, breads_seen = 0; 
   bool found = false;
   int i;
   for (i = 0; i<n; i++){
      // We will find the number of different breads
      // if the cow interval ends at i.
      // So, remove all cows that further that len units
      // from the i-th cow. The interval becomes: [left, i]
      while(K[i].x - K[lef].x > len) {
         A[K[lef].r]--; // Decrease bread count.
         // If this was the last cow of the bread, decrease
         // the number of different cows.
         if (A[K[lef].r] == 0) breads_seen--;
         // remove the cow.
         lef++;
      }
      //  Add the i-th cow and update breads seen.
      if (A[K[i].r] == 0) breads_seen++;
      A[K[i].r]++;
      // If we have seen all breads, we are done.
      if (breads_seen == num_breads) {
         found = true; ++i;
         break;
      }
   }
   // Cleanup the counting array.
   while(i > lef) {
      --A[K[lef].r];
      ++lef;
   }
   return found;
}

int main(){
   freopen("I.","r",stdin);
   freopen("O.","w",stdout);
   scanf("%d", &n);
   for (int i = 0; i<n; i++){
      scanf("%d%d", &K[i].x, &K[i].r);
   }
   // Sort by bread and translate breads in [0, n).
   sort(K, K+n, [](const cow& c1, const cow& c2) { return c1.r < c2.r; });
   int prev = -1, cur_id = -1;
   for (int i = 0; i < n; ++i) {
      int old = K[i].r;
      if (K[i].r == prev) K[i].r = cur_id;
      else {
         prev = K[i].r;
         K[i].r = ++cur_id;
      }
   }
   num_breads = cur_id + 1;
   // Sort breads by position.
   sort(K, K+n);
   int st = 0, en = K[n-1].x - K[0].x, mn;
   while (st < en) {
      mn = (st+en)/2;
      if (F(mn) == true) en = mn;
      else st = mn + 1;
   }
   printf("%d\n", st);
   return 0;
}
